/* gEDA - GNU Electronic Design Automation
 * gpcb - GNU Printed Circuit Board layout
 * Copyright (C) 1998 Ales V. Hvezda
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <config.h>
#include <stdio.h>
#include <signal.h>
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>

#include <guile/gh.h>

#include "struct.h"
#include "globals.h"
#include "defines.h"

#include "prototype.h"
#include "../libgeda/prototype.h"

#include "a_memory.h"

void
gschem_quit(void)
{
	s_clib_cache_free();
        s_clib_free();
        s_slib_free();
        /* o_text_freeallfonts();*/
        s_attrib_free();	
	s_papersizes_free();

	/* x_window_free_head(); can't do this since it causes a condition */
	/* in which window_head->... is still being refered after this */

	gtk_main_quit();
}

void 
main_prog(int argc, char *argv[])
{
	int i;
	TOPLEVEL *w_current;
	char input_str[256]; /* size is hack */
	int argv_index;
        int first_page=1;

	gtk_init(&argc, &argv);	
	visual = gdk_visual_get_system ();

	argv_index = parse_commandline(argc, argv);

	/* create log file right away */
	/* even if logging is enabled */
	s_log_init("gpcb.log");

	s_log_message("gEDA: gpcb version %s - THIS IS AN PRE-ALPHA RELEASE!\n", VERSION);

if (!quiet_mode) {
	fprintf(stderr, "THIS IS AN PRE-ALPHA RELEASE! version %s\n", VERSION);
	fprintf(stderr, "This is the first release of gpcb.\n");
}

	/* register guile (scheme) functions */
	g_register_funcs();

	s_clib_init();
 	s_slib_init();
	s_attrib_init();



        colormap = gdk_colormap_get_system (); 
	x_window_setup_colors();

	x_window_add_head();

	w_current = x_window_create_new();

	/* o_text_init(); goes away */
	o_ntext_init();
	x_repaint_background(w_current);

	i = argv_index;
	while (argv[i] != NULL) {
		if (first_page) {

			if (w_current->page_current->page_filename) {
				free(w_current->page_current->page_filename);
			}

		/* Page structure has already been created... */	
		/* so, just set the filename and open the schematic */
		/* for the first page */
			w_current->page_current->page_filename = malloc(
					sizeof(char)*strlen(argv[i])+5);
                	strcpy(w_current->page_current->page_filename, argv[i]);

			w_current->DONT_REDRAW = 1;
			net_conn_recalc_func = o_net_conn_recalc_only;
if (!quiet_mode) {
			printf("Loading pcb [%s]\n", argv[i]);
}
                	f_open(w_current, w_current->page_current->page_filename);
			i_set_filename(w_current, w_current->page_current->page_filename);

			a_zoom_limits(w_current, w_current->page_current->object_head);
			/* now update the scrollbars */
			x_hscrollbar_update(w_current);
			x_vscrollbar_update(w_current);

			first_page = 0;
		} else {
			/* Much simpler	*/
			s_page_new(w_current, argv[i]);
if (!quiet_mode) {
			printf("Loading pcb [%s]\n", argv[i]);
}
                	f_open(w_current, w_current->page_current->page_filename);
			i_set_filename(w_current, w_current->page_current->page_filename);
			a_zoom_limits(w_current, w_current->page_current->object_head);
			/* now update the scrollbars */
			x_hscrollbar_update(w_current);
			x_vscrollbar_update(w_current);
		}
		i++;
	}


	if (argv[argv_index] == NULL) {

		if (w_current->page_current->page_filename) {
			free(w_current->page_current->page_filename);
		}

		getcwd(w_current->cwd, 256);

		w_current->page_current->page_filename = malloc(
			sizeof(char)*(
				strlen(w_current->cwd)+
				strlen(w_current->untitled_name)+
                       		strlen("/_##########.sch")+1));

		w_current->num_untitled++;
		sprintf(w_current->page_current->page_filename, 
			"%s/%s_%d.sch", 
				w_current->cwd, 
				w_current->untitled_name, 
				w_current->num_untitled);

		i_set_filename(w_current, w_current->page_current->page_filename);
		
		/* update the scrollbars */
		x_hscrollbar_update(w_current); 
		x_vscrollbar_update(w_current);
        }               

	w_current->DONT_REDRAW = 0;
/*	net_conn_recalc_func = o_net_conn_recalc_draw; */

	o_redraw_all(w_current);
#if DEBUG
	gh_eval_str ("(display \"hello guile\n\")");
#endif

	if (w_current->scheme_directory == NULL) {
		fprintf(stderr, "Scheme directory NOT set!\n");
		exit(-1);
	}	

	sprintf(input_str, "%s/gpcb.scm", w_current->scheme_directory);
	g_read_file(input_str);
	
	/* enter main loop */
        gtk_main();
}

int	 
main (int argc, char *argv[])
{
/*  current_page = new_page(); */

  gh_enter (argc, argv, main_prog);
  return 0;
}
