/* gEDA - GNU Electronic Design Automation
 * gschem/gpcb - GNU Schematic Capture and Printed Circuit Board
 * Copyright (C) 1998 Ales V. Hvezda
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <config.h>
#include <stdio.h>
#include <math.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>

#include <guile/gh.h>

#include "struct.h"
#include "defines.h"
#include "globals.h"
#include "s_passing.h"
#include "o_types.h"

#include "colors.h"

#include "prototype.h"
#include "../libgeda/prototype.h"

/* W A R N I N G */
/* W A R N I N G */
/* W A R N I N G */
/* W A R N I N G */
/* W A R N I N G */
/* This file is out of sync with libgeda. */
/* circle objects do not use line_points anymore */
/* they use the CIRCLE construct */
/* to fix this: sync up files or remove this file */
/* W A R N I N G */
/* W A R N I N G */
/* W A R N I N G */
/* W A R N I N G */
/* W A R N I N G */


void
o_circle_draw(TOPLEVEL *w_current, OBJECT *o_current)
{
	int wleft, wright, wtop, wbottom; /* world bounds */
	int temp;


	if (o_current->line_points == NULL) {
		return;
	}

	
	o_circle_recalc(w_current, o_current);

	/* Get read to check for visibility of this line by using it's */
        /* bounding box */
#if 0 /* commented out because it is out of sync with libgeda */
        world_get_circle_bounds(w_current, o_current->line_points, &wleft, &wtop, &wright, &wbottom);

        if (!visible(w_current, wleft, wtop, wright, wbottom)) {
                return;
        }
#endif

	temp = dist(o_current->line_points->screen_x1,
		    o_current->line_points->screen_y1,
		    o_current->line_points->screen_x2,
		    o_current->line_points->screen_y2);

#if 1
	printf("drawing circle\n\n");
#endif

	/* To draw be sure to setup width height */
	if (w_current->override_color != -1 ) {
	gdk_gc_set_foreground(w_current->gc, 
			x_get_color(w_current->override_color));
	gdk_draw_arc(w_current->window, w_current->gc, FALSE,
				       o_current->line_points->screen_x1, 
				       o_current->line_points->screen_y1,
				       temp, temp, 0, FULL_CIRCLE);
	gdk_draw_arc(w_current->backingstore, w_current->gc, FALSE,
				       o_current->line_points->screen_x1, 
				       o_current->line_points->screen_y1,
				       temp, temp, 0, FULL_CIRCLE);
	} else {
	gdk_gc_set_foreground(w_current->gc, 
			x_get_color(o_current->color));
	gdk_draw_arc(w_current->window, w_current->gc, FALSE,
				       o_current->line_points->screen_x1, 
				       o_current->line_points->screen_y1,
				       temp, temp, 0, FULL_CIRCLE);
	gdk_draw_arc(w_current->backingstore, w_current->gc, FALSE,
				       o_current->line_points->screen_x1, 
				       o_current->line_points->screen_y1,
				       temp, temp, 0, FULL_CIRCLE);
				/* fix for aspect ? hack */
	}

				
}

/* add in offsets, get rid of global diffs_x,y */
void
o_circle_draw_xor(TOPLEVEL *w_current, int dx, int dy, OBJECT *o_current)
{
	int temp;

	if (o_current->line_points == NULL) {
		return;
	}

	temp = dist(o_current->line_points->screen_x1,
		    o_current->line_points->screen_y1,
		    o_current->line_points->screen_x2,
		    o_current->line_points->screen_y2);

	/* To draw be sure to setup width height */
	gdk_gc_set_foreground(w_current->outline_xor_gc, 
			x_get_darkcolor(o_current->color));
	gdk_draw_arc(w_current->window, w_current->outline_xor_gc, 
			       FALSE,
			       o_current->line_points->screen_x1+dx,
			       o_current->line_points->screen_y1+dy,
			       temp, temp, 0, FULL_CIRCLE);

	/* backing store ?  not appropriate here */
}

           
void
o_circle_start(TOPLEVEL *w_current, int x, int y)
{
	int temp;

	w_current->last_x = w_current->start_x = fix_x(w_current, x);
	w_current->last_y = w_current->start_y = fix_y(w_current, y);

	/* check of < 0 width and height !!!!!!!!!! hack */
	gdk_gc_set_foreground(w_current->xor_gc, 
			x_get_color(w_current->select_color));
	temp = dist(w_current->start_x, w_current->start_y, 
		    w_current->last_x, w_current->last_y);	
	gdk_draw_arc(w_current->window, w_current->xor_gc, 
		     FALSE, w_current->start_x, w_current->start_y, 
		     temp, temp, 0, FULL_CIRCLE);
}

void
o_circle_end(TOPLEVEL *w_current, int x, int y)
{
	int x1, y1;
	int x2, y2;
	int temp;

	if (w_current->inside_action == 0) {
                o_redraw(w_current, w_current->page_current->object_head);
                return;
        }

	w_current->last_x = fix_x(w_current, x);
	w_current->last_y = fix_y(w_current, y);

	/* check of < 0 width and height !!!!!!!!!! hack */

	temp = dist(w_current->start_x, w_current->start_y, 
		    w_current->last_x, w_current->last_y);	

	if (temp != 0) {
		gdk_gc_set_foreground(w_current->xor_gc, 
				x_get_color(w_current->select_color));
		gdk_draw_arc(w_current->window, w_current->xor_gc, 
			     FALSE, w_current->start_x, w_current->start_y, 
			     temp, temp, 0, FULL_CIRCLE);
	} else {
		w_current->start_x = (-1);
        	w_current->start_y = (-1);
        	w_current->last_x = (-1);
        	w_current->last_y = (-1);
		return;
	}

	gdk_gc_set_foreground(w_current->gc, 
			x_get_color(w_current->graphic_color));
	temp = dist(w_current->start_x, w_current->start_y, 
		    w_current->last_x, w_current->last_y);	
	gdk_draw_arc(w_current->window, w_current->gc, FALSE, 
		     w_current->start_x, w_current->start_y, 
		     temp, temp, 0, FULL_CIRCLE);
	gdk_draw_arc(w_current->backingstore, w_current->gc, 
		     FALSE, w_current->start_x, w_current->start_y, 
		     temp, temp, 0, FULL_CIRCLE);

	SCREENtoWORLD(w_current, w_current->start_x, w_current->start_y, &x1, &y1); 
	SCREENtoWORLD(w_current, w_current->last_x, w_current->last_y, &x2, &y2); 
	x1 = snap_grid(w_current, x1);
	y1 = snap_grid(w_current, y1);
	x2 = snap_grid(w_current, x2);
	y2 = snap_grid(w_current, x2);

/*	w_current->page_current->object_tail = o_circle_add(w_current, w_current->page_current->object_tail, */
/*		 OBJ_CIRCLE, w_current->graphic_color, x1, y1, x2, y2); */

	w_current->start_x = (-1);
        w_current->start_y = (-1);
        w_current->last_x = (-1);
        w_current->last_y = (-1);
	w_current->page_current->CHANGED=1;
}

void
o_circle_rubbercircle(TOPLEVEL *w_current, int x, int y)
{
	int diff_x, diff_y;
	int temp;

	if (w_current->inside_action == 0) {
		o_redraw(w_current, w_current->page_current->object_head);
		return;
	}

	gdk_gc_set_foreground(w_current->xor_gc,
		 x_get_color(w_current->select_color));

	temp = dist(w_current->start_x, w_current->start_y, 
		    w_current->last_x, w_current->last_y);	
	/* check of < 0 width and height !!!!!!!!!! hack */
	gdk_draw_arc(w_current->window, w_current->xor_gc, 
		FALSE, w_current->start_x, w_current->start_y, 
		temp, temp, 0, FULL_CIRCLE);

	w_current->last_x = fix_x(w_current, x);
        w_current->last_y = fix_y(w_current, y);

	diff_x = abs(w_current->last_x - w_current->start_x);
	diff_y = abs(w_current->last_y - w_current->start_y);

/*	if (diff_x >= diff_y) {
		last_y = start_y;	
	} else {
		last_x = start_x;
	}*/

	gdk_gc_set_foreground(w_current->xor_gc, 
				x_get_color(w_current->select_color));
	temp = dist(w_current->start_x, w_current->start_y, 
		    w_current->last_x, w_current->last_y);	
	gdk_draw_arc(w_current->window, w_current->xor_gc, 
		     FALSE, w_current->start_x, w_current->start_y, 
		     temp, temp, 0, FULL_CIRCLE);
}

