/* gEDA - GNU Electronic Design Automation
 * gschem/gpcb - GNU Schematic Capture and Printed Circuit Board
 * Copyright (C) 1998 Ales V. Hvezda
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <config.h>
#include <stdio.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>

#include <guile/gh.h>

#include "struct.h"
#include "defines.h"
#include "globals.h"
#include "s_passing.h"
#include "o_types.h"
#include "x_states.h"
#include "colors.h"

#include "prototype.h"
#include "../libgeda/prototype.h"


/* break with the tradition here and input a list */
/* probably should go back and do the same for o_copy o_move o_delete... */
/* hack */
void
o_edit(TOPLEVEL *w_current, OBJECT *list)
{
	/* shouldn't happen */
	if (list == NULL) { 
		/* this is an error condition hack */
		w_current->event_state = SELECT;
		i_update_status(w_current, "Select Mode");
		w_current->inside_action = 0;
		return;
	}
		
	/* for now deal with only the first item */
	switch(list->type) {

		case(OBJ_NTEXT):
			o_ntext_edit(w_current, list);
		break;
	}

	/* has to be more extensive in the future */
	/* some sort of redrawing? */
}



/* This locks the entire selected list.  It does lock components, but does NOT
change the color (of primatives of the components) though */
/* this cannot be called recursively */
void
o_lock(TOPLEVEL *w_current)
{
	OBJECT *real=NULL;
	OBJECT *o_current=NULL;

	o_current = w_current->page_current->selection_head->next;

	while(o_current != NULL) {
	
		real = (OBJECT *) o_list_search(w_current->page_current->object_head, o_current);
		if (real) {

			/* check to see if saved_color is already being used */
			if (real->saved_color == -1) {
				real->sel_func = NULL;
				real->saved_color = real->color;
				real->color = GREY;
				w_current->page_current->CHANGED=1;
			}
		}

		o_current=o_current->next;
	}

	o_unselect_all(w_current);
}


/* You can unlock something by selecting it with a bounding box... */
/* this will probably change in the future, but for now it's a something.. */
/* :-) */
/* this cannot be called recursively */
void
o_unlock(TOPLEVEL *w_current)
{
	OBJECT *real=NULL;
	OBJECT *o_current=NULL;

	o_current = w_current->page_current->selection_head->next;

	while(o_current != NULL) {
	
		real = (OBJECT *) o_list_search(w_current->page_current->object_head, o_current);
		if (real) {
			/* only unlock if sel_func is not set to something */
			if (real->sel_func == NULL) {
/*				real->sel_func = o_select; */
				real->color = real->saved_color;
				real->saved_color = -1;
				w_current->page_current->CHANGED=1;
			}
		}

		o_current=o_current->next;
	}
}

void
o_rotate_90(TOPLEVEL *w_current, OBJECT *list, int centerx, int centery)
{
	OBJECT *real;
	OBJECT *o_current;
	OBJECT *temp=NULL;
	OBJECT *selection_list=NULL;
	int new_next=0;
	int new_angle;

	/* this is okay if you just hit rotate and have nothing selected */
	if (list == NULL) { 
		w_current->event_state = SELECT;
		i_update_status(w_current, "Select Mode");
		w_current->inside_action = 0;
		return;
	}


	o_current = list;

	while (o_current != NULL) {


		/* first get the real object */	
		real = (OBJECT *) 
			o_list_search(w_current->page_current->object_head, 
					o_current);

		if (real == NULL) {
			printf("Oops! you tried to rotate an object which doesn't exists\n");
			return;	
		}


		switch(real->type) {

			case(OBJ_LINE):
				/* erase the current selection */
				w_current->override_color =
                                        w_current->background_color;
                                o_line_draw(w_current, o_current);
                                w_current->override_color = -1;

		 		o_line_rotate(w_current, 
					centerx, centery, 90, real);

				o_line_rotate(w_current, 
					centerx, centery, 90, o_current);

				o_line_draw(w_current, real);
			break;

			/* need to recalculate nets */
			case(OBJ_NET):
				/* erase the current selection */
				w_current->override_color =
                                        w_current->background_color;
                                o_net_draw(w_current, o_current);
				o_net_conn_erase(w_current, real, 0);
				o_net_conn_erase(w_current, real, 1);
                                w_current->override_color = -1;

		 		o_net_rotate(w_current, 
					centerx, centery, 90, real);

				o_net_rotate(w_current, 
					centerx, centery, 90, o_current);

				o_net_draw(w_current, real);
			break;

			case(OBJ_NTEXT):
				/* erase the current selection */
				w_current->override_color =
                                        w_current->background_color;
                                o_ntext_draw(w_current, o_current);
                                w_current->override_color = -1;

				new_angle = ( real->angle + 90 ) % 360;
		 		o_ntext_rotate(w_current, 
				/* 	o_current->x, o_current->y,*/
					centerx, centery, 
					new_angle, 90, real);

				new_angle = ( o_current->angle + 90 ) % 360;
				o_ntext_rotate(w_current, 
					centerx, centery,
				/* 	o_current->x, o_current->y,*/
					new_angle, 90, o_current);

				o_ntext_draw(w_current, real);
			break;
		}


		if (new_next) {
			o_current = temp;
			temp = NULL;
			new_next = 0;
		} else {
			o_current = o_current->next;
		}
	}

	w_current->page_current->CHANGED=1;
	
	/* are the objects to be appended to the selection list */
	if (selection_list) {
		selection_list = (OBJECT *) return_head(selection_list);
		
		selection_list->prev = w_current->page_current->selection_tail;
		w_current->page_current->selection_tail->next = selection_list;
		w_current->page_current->selection_tail = return_tail(
		                    w_current->page_current->selection_head);	 
	}
		
	o_conn_recalc_all(w_current, w_current->page_current->object_head);
	o_redraw_selected(w_current); 
}

void
o_embed(TOPLEVEL *w_current)
{
	OBJECT *real=NULL;
	char *new_basename;
	OBJECT *o_current=NULL;

	o_current = w_current->page_current->selection_head->next;

	while(o_current != NULL) {
	
		real = (OBJECT *) o_list_search(w_current->page_current->object_head, o_current);
		if (real) {
		 if (real->type == OBJ_COMPLEX) {
		   if (strncmp(real->complex_clib, "EMBEDDED", 8) != 0) {

			if (real->complex_clib) 
				free(real->complex_clib);
		
	/* this code fragment needs to be cleaned up... way too many */
	/* mallocs frees and strcpy's hack */	
			real->complex_clib = (char *) malloc(sizeof(char)*(
						strlen("EMBEDDED")+1));
							     
			strcpy(real->complex_clib, "EMBEDDED");
			
			
			new_basename = (char *) malloc(sizeof(char)*(
						strlen("EMBEDDED")+
						strlen(real->complex_basename)+	
						1));
			
			sprintf(new_basename, "EMBEDDED%s", 
						real->complex_basename);
			
			free(real->complex_basename);
			
			real->complex_basename = (char *) malloc(sizeof(char)*(
						strlen(new_basename)+1));	
			
			strcpy(real->complex_basename, new_basename);
			
			free(new_basename);

			w_current->page_current->CHANGED=1;
		    }
		  }
		}
	         

		o_current=o_current->next;
	}
}

void
o_unembed(TOPLEVEL *w_current)
{
	OBJECT *real=NULL;
	char *new_basename;
	char *new_clib;
	OBJECT *o_current=NULL;

	o_current = w_current->page_current->selection_head->next;

	while(o_current != NULL) {
	
		real = (OBJECT *) o_list_search(w_current->page_current->object_head, o_current);
		if (real) {
		 if (real->type == OBJ_COMPLEX) {
		   if (strncmp(real->complex_clib, "EMBEDDED", 8) == 0) {
			
			new_basename = (char *) malloc(sizeof(char)*(
						strlen(real->complex_basename)+
						1));
			
			sprintf(new_basename, "%s", (real->complex_basename+8));
			
		
			new_clib = (char *) s_clib_search(new_basename);
			
			if (!new_clib) {
				fprintf(stderr, "Could not find component [%s], while trying to unembed.\n", real->complex_basename);
				fprintf(stderr, "Component is still embedded\n");
			} else {

				free(real->complex_basename);
			
				real->complex_basename = new_basename;

				free(real->complex_clib);
			
				real->complex_clib = new_clib;	

				w_current->page_current->CHANGED=1;
			}
		    }
	  	  }
		}

		o_current=o_current->next;
	}
}

void
o_mirror(TOPLEVEL *w_current, OBJECT *list, int centerx, int centery)
{
	OBJECT *real;
	OBJECT *o_current;
	OBJECT *temp=NULL;
	OBJECT *selection_list=NULL;
	int new_next=0;

printf("o_mirror\n");
	if (list == NULL) { 
		w_current->event_state = SELECT;
		i_update_status(w_current, "Select Mode");
		w_current->inside_action = 0;
		return;
	}


	o_current = list;

	while (o_current != NULL) {


		/* first get the real object */	
		real = (OBJECT *) 
			o_list_search(w_current->page_current->object_head, 
					o_current);

		if (real == NULL) {
			printf("Oops! you tried to mirror an object which doesn't exists\n");
			return;	
		}


		switch(real->type) {

			case(OBJ_LINE):
				/* erase the current selection */
				w_current->override_color =
                                        w_current->background_color;
                                o_line_draw(w_current, o_current);
                                w_current->override_color = -1;

		 		o_line_mirror(w_current, 
					centerx, centery, real);

				o_line_mirror(w_current, 
					centerx, centery, o_current);

				o_line_draw(w_current, real);
			break;

			/* need to recalculate nets */
			case(OBJ_NET):
				/* erase the current selection */
				w_current->override_color =
                                        w_current->background_color;
                                o_net_draw(w_current, o_current);
				o_net_conn_erase(w_current, real, 0);
				o_net_conn_erase(w_current, real, 1);
                                w_current->override_color = -1;

		 		o_net_mirror(w_current, 
					centerx, centery, real);

				o_net_mirror(w_current, 
					centerx, centery, o_current);

				o_net_draw(w_current, real);
			break;

			case(OBJ_COMPLEX):

				if (strncmp(real->complex_clib, 
					"EMBEDDED", 8) == 0) {
					s_log_message("Mirroring of embedded components not supported yet\n");
						
				} else {
				/* component is not embedded */

				/* crude hack */
				if (real->angle == 90 || 
					real->angle == 270) {

			
			 		temp = o_complex_mirror2(w_current,  
						w_current->page_current->object_tail,
						centerx, centery, real);
				
					/* erase the current selection */
					w_current->override_color =
               	                		w_current->background_color;
               	        		o_complex_draw(w_current, o_current);
               	                	w_current->override_color = -1;
					o_complex_draw(w_current, temp);

					selection_list = (OBJECT *) 
						o_list_copy_to(w_current, 
							selection_list, 
							temp, SELECTION); 

					new_next = TRUE;
					temp = o_current->next;

					s_delete(w_current, o_current);

					w_current->page_current->
					   selection_tail = return_tail(
				     		w_current->page_current->
							selection_head);

				} else {

					/* erase the current selection */
					w_current->override_color =
                                       	 w_current->background_color;
                                	o_complex_draw(w_current, o_current);
                                	w_current->override_color = -1;

					w_current->ADDING_SEL=1;
		 			o_complex_mirror(w_current, 
						centerx, centery, real);

					o_complex_mirror(w_current, 
						centerx, centery, o_current);
					w_current->ADDING_SEL=0;

					o_complex_draw(w_current, real);
				}
			    }		
			break;

			case(OBJ_NTEXT):
				/* erase the current selection */
				w_current->override_color =
                                        w_current->background_color;
                                o_ntext_draw(w_current, o_current);
                                w_current->override_color = -1;

		 		o_ntext_mirror(w_current, 
					centerx, centery, 
					real);

				w_current->ADDING_SEL=1;
				o_ntext_mirror(w_current, 
					centerx, centery,
					o_current);
				w_current->ADDING_SEL=0;

				o_ntext_draw(w_current, real);
			break;
		}


 		if (new_next) {
			o_current = temp;
			temp = NULL;
			new_next = 0;
		} else {
			o_current = o_current->next;
		}

	}

	w_current->page_current->CHANGED=1;
	
	/* are the objects to be appended to the selection list */
	if (selection_list) {
		selection_list = (OBJECT *) return_head(selection_list);
		
		selection_list->prev = w_current->page_current->selection_tail;
		w_current->page_current->selection_tail->next = selection_list;
		w_current->page_current->selection_tail = return_tail(
		                    w_current->page_current->selection_head);	 
	}
		
	o_conn_recalc_all(w_current, w_current->page_current->object_head);
	o_redraw_selected(w_current); 
}

