/* gEDA - GNU Electronic Design Automation
 * gschem/gpcb - GNU Schematic Capture and Printed Circuit Board
 * Copyright (C) 1998 Ales V. Hvezda
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <config.h>
#include <stdio.h>
#include <math.h>

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gdk/gdkx.h>

#include <guile/gh.h>


#include "struct.h"
#include "defines.h"
#include "globals.h"
#include "s_passing.h"
#include "o_types.h"
#include "x_states.h"

#include "colors.h"

#include "prototype.h"
#include "../libgeda/prototype.h"


void
o_net_conn_erase(TOPLEVEL *w_current, OBJECT *o_current, int whichone)
{
	
	gdk_gc_set_foreground(w_current->gc, 
			x_get_color(w_current->background_color) );

	if (whichone == 1) { /* first */
		o_conn_draw_endpoint(w_current, w_current->gc, o_current->line_points->screen_x1,
					  o_current->line_points->screen_y1);
	} else if (whichone == 2) { /* second */
		o_conn_draw_endpoint(w_current, w_current->gc, o_current->line_points->screen_x2,
					  o_current->line_points->screen_y2);
	} else if (whichone == 3) { /* round first points */
		o_conn_draw_midpoint(w_current, w_current->gc, o_current->line_points->screen_x1,
					  o_current->line_points->screen_y1);
	} else if (whichone == 4) { /* round second points */
		o_conn_draw_midpoint(w_current, w_current->gc, o_current->line_points->screen_x2,
					  o_current->line_points->screen_y2);
	}

	/* now draw the line */
	(*o_current->draw_func)(w_current, o_current); /* try */
}

void
o_net_conn_recalc_draw(TOPLEVEL *w_current, OBJECT *object_list)
{

	/* change the name object_list hack */
	OBJECT *connection=NULL;
	OBJECT *prev_conn=NULL;
	int whichone;
	int changed;

	changed = w_current->FORCE_CONN_UPDATE;

	if (object_list == NULL)
		return;

	object_list->connected_to_1 = NULL;
	object_list->connected_to_2 = NULL;


	object_list->connection_1 = CONNECTION_REGULAR;
	object_list->connection_2 = CONNECTION_REGULAR;

	connection = (OBJECT *) o_conn_search(
			object_list->line_points->x1, 
			object_list->line_points->y1, 
			object_list->sid, w_current->page_current->object_head, 
			&whichone);

#if DEBUG
	printf("1 which one %d\n", whichone);
#endif
	prev_conn = object_list->connected_to_1;
	/* do one */
	object_list->connected_to_1 = connection;
	if (whichone == 1) {
		if (connection->connected_to_1 != object_list) {
			o_net_conn_erase(w_current, connection, whichone);

			/* was after the first draw func */
			connection->connected_to_1 = object_list;

			(*connection->draw_func)(w_current, connection);
			o_net_conn_erase(w_current, connection, whichone); /* was outside */
			/* this one was outside of this if */
			object_list->connection_1 = CONNECTION_REGULAR;

			changed = 1;
		} /* linking inside or out? */

	} else if (whichone == 2) {
		if (connection->connected_to_2 != object_list) {
			o_net_conn_erase(w_current, connection, whichone);

			/* was after the first draw func */
			connection->connected_to_2 = object_list;

			(*connection->draw_func)(w_current, connection);
			o_net_conn_erase(w_current, connection, whichone); /* was outside */
			/* this one was outside of this if */
			object_list->connection_1 = CONNECTION_REGULAR;

			changed = 1;
		} /* linking inside or out? */

	} else if (whichone == 3) {

			/*printf("First connection connected to middle of %s\n", connection->name);*/
                        object_list->connection_1 = CONNECTION_ROUND;


                        /* 3 means first round connection */
                        /* use a define for crying out loud... hack */
			/* why is this done? hack */
			/* major bug if it's in, extra erasing */
                        /*o_net_conn_erase(w_current, connection, 3);*/

                        changed = 1;

	}

	/* do two */
	connection = (OBJECT *) o_conn_search(
			object_list->line_points->x2,
		        object_list->line_points->y2, 
			object_list->sid, w_current->page_current->object_head, 
			&whichone);

#if DEBUG
	printf("2 which one %d\n", whichone);
#endif

	prev_conn = object_list->connected_to_2;
	object_list->connected_to_2 = connection;
	if (whichone == 1) {

		if (connection->connected_to_1 != object_list) {
			o_net_conn_erase(w_current, connection, whichone);

			/* was after the first draw func */
			connection->connected_to_1 = object_list;

			(*connection->draw_func)(w_current, connection);
			o_net_conn_erase(w_current, connection, whichone); /* was outside */
			/* this one was outside of this if */
			object_list->connection_2 = CONNECTION_REGULAR;

			changed = 1;
		} /* the linking inside or out of this if? */ 

	} else if (whichone == 2) {
		if (connection->connected_to_2 != object_list) {
			o_net_conn_erase(w_current, connection, whichone);

			/* was after the first draw func */
			(*connection->draw_func)(w_current, connection);

			connection->connected_to_2 = object_list;
			o_net_conn_erase(w_current, connection, whichone); /* was outside */
			/* this one was outside of this if */
			object_list->connection_2 = CONNECTION_REGULAR;

			changed = 1;
		} /* the linking inside or out of this if? */ 

	} else if (whichone == 3) {

			/*printf("Second connection connected to middle of %s\n", connection->name);*/

                        /* use a define for crying out loud... hack */
                        object_list->connection_2 = CONNECTION_ROUND;

                        /* 4 means second round connection */

			/* why is this done? hack */
			/* major bug if it's in, extra erasing */
                        /*o_net_conn_erase(w_current, connection, 4);*/

	}

	/* this in, but I found that if you deleted a complex from multi nets 
	   then some of the nets didn't get redrawn */
	/*if (changed)*/

	(*object_list->draw_func)(w_current, object_list); 

	changed = 0; /* blah don't need this hack */
}

void
o_net_draw(TOPLEVEL *w_current, OBJECT *o_current)
{
	int size;

	if (o_current == NULL) {
		return;
	}

	if (o_current->line_points == NULL) {
		return;
	}

	o_net_recalc(w_current, o_current);


	/* reuse line's routine */
/*        if (!o_line_visible(w_current, o_current->line_points)) { */
/*                return; */
/*        } */


#if DEBUG 
	printf("drawing net\n\n");
#endif

	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		size = return_zoom_number(w_current->page_current->zoom_factor);

		if (size < 0) 
			size=0;

		gdk_gc_set_line_attributes(w_current->gc, size, GDK_LINE_SOLID,
				GDK_CAP_NOT_LAST,
				GDK_JOIN_MITER);
	}  

	if (w_current->override_color != -1 ) {
	gdk_gc_set_foreground(w_current->gc, 
			x_get_color(w_current->override_color));
	gdk_draw_line(w_current->window, w_current->gc, 
				       o_current->line_points->screen_x1, 
				       o_current->line_points->screen_y1,
				       o_current->line_points->screen_x2,
				       o_current->line_points->screen_y2);
	gdk_draw_line(w_current->backingstore, w_current->gc, 
				       o_current->line_points->screen_x1, 
				       o_current->line_points->screen_y1,
				       o_current->line_points->screen_x2,
				       o_current->line_points->screen_y2);
	} else {
	gdk_gc_set_foreground(w_current->gc, 
			x_get_color(o_current->color));
	gdk_draw_line(w_current->window, w_current->gc, 
				       o_current->line_points->screen_x1, 
				       o_current->line_points->screen_y1,
				       o_current->line_points->screen_x2,
				       o_current->line_points->screen_y2);
	gdk_draw_line(w_current->backingstore, w_current->gc, 
				       o_current->line_points->screen_x1, 
				       o_current->line_points->screen_y1,
				       o_current->line_points->screen_x2,
				       o_current->line_points->screen_y2);
	}

        /* yes zero is right for the width -> use hardware lines */
	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		gdk_gc_set_line_attributes(w_current->gc, 0, GDK_LINE_SOLID,
				GDK_CAP_NOT_LAST,
				GDK_JOIN_MITER);
	}

	/* only draw the connection points, if: */
	/* - you are drawing regular lines, */
	/* - you aren't redrawing selected (DONT_DRAW_CONN), */ 
	/* - And, you are erasing them */
	if ( ((w_current->override_color == -1) && (!w_current->DONT_DRAW_CONN)) || 
			(w_current->override_color == w_current->background_color ) ) {
			/* I think the above is right ... check it hack */

		if (w_current->override_color != -1) {
			gdk_gc_set_foreground(w_current->gc, 
				x_get_color(w_current->override_color));
		} else {
			gdk_gc_set_foreground(w_current->gc, 
				x_get_color(w_current->net_endpoint_color));
		}

		if (!(o_current->connected_to_1) &&
			(o_current->connection_1 == CONNECTION_REGULAR) ){
				o_conn_draw_endpoint(w_current, w_current->gc, 
					  o_current->line_points->screen_x1,
					  o_current->line_points->screen_y1);
		} else if ( (o_current->connected_to_1) && 
			(o_current->connection_1 == CONNECTION_ROUND) ) {
				o_conn_draw_midpoint(w_current, w_current->gc, 
					  o_current->line_points->screen_x1,
					  o_current->line_points->screen_y1);

		}

		if (!(o_current->connected_to_2) &&
			(o_current->connection_2 == CONNECTION_REGULAR) ) {
				o_conn_draw_endpoint(w_current, w_current->gc, 
					  o_current->line_points->screen_x2,
					  o_current->line_points->screen_y2);
		} else if ( (o_current->connected_to_2) && 
			(o_current->connection_2 == CONNECTION_ROUND) ) {
				o_conn_draw_midpoint(w_current, w_current->gc, 
					  o_current->line_points->screen_x2,
					  o_current->line_points->screen_y2);
		}

		/* this should only happen when erasing */
		if ( (o_current->connected_to_1 == NULL) && 
			(o_current->connection_1 == CONNECTION_ROUND ) ) {
			o_conn_draw_midpoint(w_current, w_current->gc, 
				  o_current->line_points->screen_x1,
				  o_current->line_points->screen_y1);
			/* this was a screwup???? why is this here ? */
			/* o_current->connection_1 == CONNECTION_REGULAR;*/
		}

		if ( (o_current->connected_to_2 == NULL) && 
			(o_current->connection_2 == CONNECTION_ROUND ) ) {
			o_conn_draw_midpoint(w_current, w_current->gc, 
				  o_current->line_points->screen_x2,
				  o_current->line_points->screen_y2);
			/* this was a screwup???? why is this here ? */
			/* o_current->connection_2 == CONNECTION_REGULAR;*/
		}

	/* hack make size of rectangle configurable */
	}
	

#if DEBUG
	printf("drawing net\n");
#endif
				
}

void
o_net_draw_xor(TOPLEVEL *w_current, int dx, int dy, OBJECT *o_current)
{
	int size;
        if (o_current->line_points == NULL) {
                return;
        }

        gdk_gc_set_foreground(w_current->outline_xor_gc, 
			x_get_darkcolor(o_current->color));

	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		size = return_zoom_number(w_current->page_current->zoom_factor);
		gdk_gc_set_line_attributes(w_current->outline_xor_gc, size+1,
				GDK_LINE_SOLID,
				GDK_CAP_NOT_LAST,
				GDK_JOIN_MITER);
	}   	

        gdk_draw_line(w_current->window, w_current->outline_xor_gc,
                       o_current->line_points->screen_x1+dx,
                       o_current->line_points->screen_y1+dy,
                       o_current->line_points->screen_x2+dx,
                       o_current->line_points->screen_y2+dy);

	/* backing store ? not approriate here */

	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		gdk_gc_set_line_attributes(w_current->outline_xor_gc, 0,
					GDK_LINE_SOLID,
					GDK_CAP_NOT_LAST,
					GDK_JOIN_MITER);
	}
}            


void
o_net_start(TOPLEVEL *w_current, int x, int y)
{
	int size;
	w_current->last_x = w_current->start_x = fix_x(w_current, x);
	w_current->last_y = w_current->start_y = fix_y(w_current, y);


	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		size = return_zoom_number(w_current->page_current->zoom_factor);
		gdk_gc_set_line_attributes(w_current->xor_gc, size,
				GDK_LINE_SOLID,
				GDK_CAP_NOT_LAST,
				GDK_JOIN_MITER);
	}   

	gdk_gc_set_foreground(w_current->xor_gc, 
			x_get_color(w_current->select_color) );
	gdk_draw_line(w_current->window, w_current->xor_gc, w_current->start_x, w_current->start_y, w_current->last_x, w_current->last_y);	

	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		gdk_gc_set_line_attributes(w_current->xor_gc, 0,
				GDK_LINE_SOLID,
				GDK_CAP_NOT_LAST,
				GDK_JOIN_MITER);
        } 
}

void
o_net_end(TOPLEVEL *w_current, int x, int y)
{
	int x1, y1;
	int x2, y2;
	int color;
	int size;

	if (w_current->inside_action == 0) {
                o_redraw(w_current, w_current->page_current->object_head);
                return;
        }

	if (w_current->override_net_color == -1) {
                color = w_current->net_color;
        } else {
                color = w_current->override_net_color;
        }          

	size = return_zoom_number(w_current->page_current->zoom_factor);

	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		gdk_gc_set_line_attributes(w_current->xor_gc, size,
					GDK_LINE_SOLID,
					GDK_CAP_NOT_LAST,
					GDK_JOIN_MITER);
        } 

	gdk_gc_set_foreground(w_current->xor_gc, 
			x_get_color(w_current->select_color) );
	gdk_draw_line(w_current->window, w_current->xor_gc, w_current->start_x, w_current->start_y, w_current->last_x, w_current->last_y);	

	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		gdk_gc_set_line_attributes(w_current->xor_gc, 0,
					GDK_LINE_SOLID,
					GDK_CAP_NOT_LAST,
					GDK_JOIN_MITER);
		gdk_gc_set_line_attributes(w_current->gc, size,
					GDK_LINE_SOLID,
					GDK_CAP_NOT_LAST,
					GDK_JOIN_MITER);
	}      


        /* don't allow zero length nets */
	/* this ends the net drawing behavior we want this? hack */
        if ( (w_current->start_x == w_current->last_x) && 
             (w_current->start_y == w_current->last_y) ) {
                w_current->start_x = (-1);
                w_current->start_y = (-1);
                w_current->last_x = (-1);
                w_current->last_y = (-1);
		w_current->inside_action=0;
                w_current->event_state = SELECT;
                i_update_status(w_current, "Select Mode");
		o_net_eraserubber(w_current);
                return;
        }


	gdk_gc_set_foreground(w_current->gc, 
			x_get_color(color));
	gdk_draw_line(w_current->window, w_current->gc, w_current->start_x, w_current->start_y, w_current->last_x, w_current->last_y);	
	gdk_draw_line(w_current->backingstore, w_current->gc, w_current->start_x, w_current->start_y, w_current->last_x, w_current->last_y);	

	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		gdk_gc_set_line_attributes(w_current->gc, 0,
				GDK_LINE_SOLID,
				GDK_CAP_NOT_LAST,
				GDK_JOIN_MITER);
        } 

	SCREENtoWORLD(w_current, w_current->start_x, w_current->start_y, &x1, &y1); 
	SCREENtoWORLD(w_current, w_current->last_x, w_current->last_y, &x2, &y2); 
	x1 = snap_grid(w_current, x1);
	y1 = snap_grid(w_current, y1);
	x2 = snap_grid(w_current, x2);
	y2 = snap_grid(w_current, y2);

	w_current->save_x = w_current->last_x;
	w_current->save_y = w_current->last_y;

	w_current->page_current->object_tail = o_net_add(w_current, w_current->page_current->object_tail, OBJ_NET, color, x1, y1, x2, y2);
	w_current->page_current->CHANGED=1;
	w_current->start_x = w_current->save_x;
        w_current->start_y = w_current->save_y;
}

void
o_net_rubbernet(TOPLEVEL *w_current, int x, int y)
{
	int diff_x, diff_y;
	int size;

	if (w_current->inside_action == 0) {
		o_redraw(w_current, w_current->page_current->object_head);
		return;
	}

	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		size = return_zoom_number(w_current->page_current->zoom_factor);
		gdk_gc_set_line_attributes(w_current->xor_gc, size,
					GDK_LINE_SOLID,
					GDK_CAP_NOT_LAST,
					GDK_JOIN_MITER);
        }  

	gdk_gc_set_foreground(w_current->xor_gc, x_get_color(w_current->select_color) );
	gdk_draw_line(w_current->window, w_current->xor_gc, w_current->start_x, w_current->start_y, w_current->last_x, w_current->last_y);	

	/* going into ortho mode (control key not pressed) */
		/* erase non-ortho line */

	/* going into non-ortho mode (control key pressed) */
		/* erase ortho line */

	w_current->last_x = fix_x(w_current, x);
        w_current->last_y = fix_y(w_current, y);


	/* If you press the control key then you can draw non-ortho nets */
	if (!w_current->CONTROLKEY) {
		diff_x = abs(w_current->last_x - w_current->start_x);
		diff_y = abs(w_current->last_y - w_current->start_y);

		if (diff_x >= diff_y) {
			w_current->last_y = w_current->start_y;	
		} else {
			w_current->last_x = w_current->start_x;
		}
	}

	gdk_gc_set_foreground(w_current->xor_gc, 
			x_get_color(w_current->select_color) );
	gdk_draw_line(w_current->window, w_current->xor_gc, w_current->start_x, w_current->start_y, w_current->last_x, w_current->last_y);	

	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		gdk_gc_set_line_attributes(w_current->xor_gc, 0,
					GDK_LINE_SOLID,
					GDK_CAP_NOT_LAST,
					GDK_JOIN_MITER);
        }
}

/* used in button cancel code in x_events.c */
void
o_net_eraserubber(TOPLEVEL *w_current)
{
	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		gdk_gc_set_line_attributes(w_current->gc, w_current->page_current->zoom_factor+1,
				GDK_LINE_SOLID,
				GDK_CAP_NOT_LAST,
				GDK_JOIN_MITER);
        }   

	gdk_gc_set_foreground(w_current->gc, 
			x_get_color(w_current->background_color) );
	gdk_draw_line(w_current->window, w_current->gc, w_current->start_x, w_current->start_y, w_current->last_x, w_current->last_y);	

	if (w_current->page_current->zoom_factor > 0 && w_current->net_style == THICK ) {
		gdk_gc_set_line_attributes(w_current->gc, 0,
				GDK_LINE_SOLID,
				GDK_CAP_NOT_LAST,
				GDK_JOIN_MITER);
	}  
}

